@@ -38,12 +38,12 @@ def repackNSBMD(workfolder, infolder, outfolder, extension=".nsbmd", readfunc=No
         if nsbmd is not None and len(nsbmd.textures) > 0:
             fixtransp = checkalpha = zerotransp = False
             if writefunc is not None:
-                fixtransp, checkalpha, zerotransp = writefunc(file, nsbmd)
+                fixtransp, checkalpha, zerotransp, backwards = writefunc(file, nsbmd)
             for texi in range(len(nsbmd.textures)):
                 pngname = file.replace(extension, "") + "_" + nsbmd.textures[texi].name + ".png"
                 if os.path.isfile(workfolder + pngname):
                     common.logDebug(" Repacking", pngname, "...")
-                    writeNSBMD(outfolder + file, nsbmd, texi, workfolder + pngname, fixtransp, checkalpha, zerotransp)
+                    writeNSBMD(outfolder + file, nsbmd, texi, workfolder + pngname, fixtransp, checkalpha, zerotransp, backwards)
     common.logMessage("Done!")
 
 
@@ -320,8 +320,8 @@ def readNARC(narcfile):
         common.logDebug("filenum:", filenum)
         for i in range(filenum):
             subfile = NARCFile()
-            subfile.start = narc.gmif + 8 + f.readUInt()
-            subfile.size = narc.gmif + 8 + f.readUInt() - subfile.start
+            subfile.start = 8 + f.readUInt()
+            subfile.size = 8 + f.readUInt() - subfile.start
             common.logDebug(vars(subfile))
             narc.files.append(subfile)
         # Read BTNF
@@ -332,6 +332,8 @@ def readNARC(narcfile):
             return None
         sectionsize = f.readUInt()
         narc.gmif = narc.btnf + sectionsize
+        for subfile in narc.files:
+            subfile.start += narc.gmif
         # TODO: handle directories
         f.seek(8, 1)
         for i in range(filenum):
@@ -1504,7 +1506,7 @@ def drawNSBMD(file, nsbmd, texi):
     img.save(file, "PNG")
 
 
-def writeNSBMD(file, nsbmd, texi, infile, fixtransp=False, checkalpha=False, zerotransp=True):
+def writeNSBMD(file, nsbmd, texi, infile, fixtransp=False, checkalpha=False, zerotransp=True, backwards=False):
     img = Image.open(infile)
     img = img.convert("RGBA")
     pixels = img.load()
@@ -1520,24 +1522,24 @@ def writeNSBMD(file, nsbmd, texi, infile, fixtransp=False, checkalpha=False, zer
         if tex.format == 1:
             for i in range(tex.height):
                 for j in range(tex.width):
-                    index = common.getPaletteIndex(paldata, pixels[j, i], fixtransp=fixtransp, checkalpha=checkalpha, zerotransp=zerotransp)
+                    index = common.getPaletteIndex(paldata, pixels[j, i], fixtransp=fixtransp, checkalpha=checkalpha, zerotransp=zerotransp, backwards=backwards)
                     alpha = (pixels[j, i][3] * 8) // 256
                     f.writeByte(index | (alpha << 5))
         # 4-color Palette
         elif tex.format == 2:
             for i in range(tex.height):
                 for j in range(0, tex.width, 4):
-                    index1 = common.getPaletteIndex(paldata, pixels[j, i], fixtransp=fixtransp, checkalpha=checkalpha, zerotransp=zerotransp)
-                    index2 = common.getPaletteIndex(paldata, pixels[j + 1, i], fixtransp=fixtransp, checkalpha=checkalpha, zerotransp=zerotransp)
-                    index3 = common.getPaletteIndex(paldata, pixels[j + 2, i], fixtransp=fixtransp, checkalpha=checkalpha, zerotransp=zerotransp)
-                    index4 = common.getPaletteIndex(paldata, pixels[j + 3, i], fixtransp=fixtransp, checkalpha=checkalpha, zerotransp=zerotransp)
+                    index1 = common.getPaletteIndex(paldata, pixels[j, i], fixtransp=fixtransp, checkalpha=checkalpha, zerotransp=zerotransp, backwards=backwards)
+                    index2 = common.getPaletteIndex(paldata, pixels[j + 1, i], fixtransp=fixtransp, checkalpha=checkalpha, zerotransp=zerotransp, backwards=backwards)
+                    index3 = common.getPaletteIndex(paldata, pixels[j + 2, i], fixtransp=fixtransp, checkalpha=checkalpha, zerotransp=zerotransp, backwards=backwards)
+                    index4 = common.getPaletteIndex(paldata, pixels[j + 3, i], fixtransp=fixtransp, checkalpha=checkalpha, zerotransp=zerotransp, backwards=backwards)
                     f.writeByte((index4 << 6) | (index3 << 4) | (index2 << 2) | index1)
         # 16/256-color Palette
         elif tex.format == 3 or tex.format == 4:
             for i in range(tex.height):
                 for j in range(0, tex.width, 2):
-                    index1 = common.getPaletteIndex(paldata, pixels[j, i], fixtransp=fixtransp, checkalpha=checkalpha, zerotransp=zerotransp)
-                    index2 = common.getPaletteIndex(paldata, pixels[j + 1, i], fixtransp=fixtransp, checkalpha=checkalpha, zerotransp=zerotransp)
+                    index1 = common.getPaletteIndex(paldata, pixels[j, i], fixtransp=fixtransp, checkalpha=checkalpha, zerotransp=zerotransp, backwards=backwards)
+                    index2 = common.getPaletteIndex(paldata, pixels[j + 1, i], fixtransp=fixtransp, checkalpha=checkalpha, zerotransp=zerotransp, backwards=backwards)
                     writeNCGRData(f, 4 if tex.format == 3 else 8, index1, index2)
         # 4x4-Texel Compressed Texture
         elif tex.format == 5:
@@ -1546,7 +1548,7 @@ def writeNSBMD(file, nsbmd, texi, infile, fixtransp=False, checkalpha=False, zer
         elif tex.format == 6:
             for i in range(tex.height):
                 for j in range(tex.width):
-                    index = common.getPaletteIndex(paldata, pixels[j, i], fixtransp=fixtransp, checkalpha=checkalpha, zerotransp=zerotransp)
+                    index = common.getPaletteIndex(paldata, pixels[j, i], fixtransp=fixtransp, checkalpha=checkalpha, zerotransp=zerotransp, backwards=backwards)
                     alpha = (pixels[j, i][3] * 32) // 256
                     f.writeByte(index | (alpha << 3))
         # Direct Color Texture
@@ -1554,6 +1556,55 @@ def writeNSBMD(file, nsbmd, texi, infile, fixtransp=False, checkalpha=False, zer
             common.logError("Texture format 7 not implemented")
 
 
+def readManualCells(manualcells):
+    ncer = NCER()
+    ncer.banks = []
+    ncer.banknum = 0
+    ncer.tbank = ncer.bankoffset = ncer.blocksize = ncer.partitionoffset = 0
+    curroff = 0
+    for manualbank in manualcells:
+        repeat = int(manualbank["repeat"]) if "repeat" in manualbank else 1
+        for i in range(repeat):
+            bank = Bank()
+            bank.cells = []
+            ncer.banks.append(bank)
+            ncer.banknum += 1
+    i = 0
+    banki = 0
+    while i < ncer.banknum:
+        manualbank = manualcells[banki]
+        repeat = int(manualbank["repeat"]) if "repeat" in manualbank else 1
+        for r in range(repeat):
+            bank = ncer.banks[i]
+            bank.cellnum = len(manualbank["cells"])
+            bank.layernum = 1
+            bank.partitionoffset = bank.width = bank.height = 0
+            for j in range(len(manualbank["cells"])):
+                manualcell = manualbank["cells"][j]
+                cell = Cell()
+                cell.objoffset = cell.layer = cell.objmode = cell.priority = 0
+                cell.mosaic = cell.depth = cell.xflip = cell.yflip = cell.rsflag = False
+                cell.width = manualcell["width"]
+                cell.height = manualcell["height"]
+                cell.pal = manualbank["pal"] if "pal" in manualbank else 0
+                cell.x = manualcell["x"] if "x" in manualcell else 0
+                cell.y = manualcell["y"] if "y" in manualcell else 0
+                if cell.x + cell.width > bank.width:
+                    bank.width = cell.x + cell.width
+                if cell.y + cell.height > bank.height:
+                    bank.height = cell.y + cell.height
+                cell.numcell = j
+                cell.tileoffset = curroff
+                curroff += ((cell.width * cell.height) // (8 * 8))
+                bank.cells.append(cell)
+            common.logDebug(vars(bank))
+            for cell in bank.cells:
+                common.logDebug(vars(cell))
+            i += 1
+        banki += 1
+    return ncer
+
+
 def readNitroGraphicNBFC(palettefile, tilefile, mapfile, lineal=False, bpp=0):
     if not os.path.isfile(palettefile):
         common.logError("Palette", palettefile, "not found")
